@@ -368,7 +368,7 @@ class SX126x :
         gpio.output(self._reset, gpio.HIGH)
         while gpio.input(self._busy) == gpio.HIGH : pass
 
-    def sleep(self, option) :
+    def sleep(self, option=SLEEP_WARM_START) :
         self._setStandby(self.STANDBY_RC)
 
     def wake(self) :
@@ -532,11 +532,26 @@ class SX126x :
             if irqStat & self.IRQ_TIMEOUT : self._status = self.STATUS_TX_TIMEOUT
             else : self._status = self.STATUS_TX_DONE
 
-    def write(self, data, length = 1) :
+    def write(self, data, length=0) :
+        if type(data) is list or type(data) is tuple :
+            if length == 0 or length > len(data) : length = len(data)
+        elif type(data) is int or type(data) is float :
+            length = 1
+            data = [int(data)]
+        else : raise TypeError("input data must be list, tuple, integer or float")
         self._writeBuffer(self._bufferIndex, data, length)
         self._bufferIndex = (self._bufferIndex + length) % 256
         self._payloadTxRx += length
 
+    def put(self, data) :
+        if type(data) is bytes or type(data) is bytearray :
+            dataList = list(data)
+            length = len(dataList)
+        else : raise TypeError("input data must be bytes or bytearray")
+        self._writeBuffer(self._bufferIndex, dataList, length)
+        self._bufferIndex = (self._bufferIndex + length) % 256
+        self._payloadTxRx += length
+
     def request(self, timeout=RX_MODE_SINGLE) :
         self._irqSetup(self.IRQ_RX_DONE | self.IRQ_TIMEOUT | self.IRQ_HEADER_ERR | self.IRQ_CRC_ERR)
         self._status = self.STATUS_RX_WAIT
@@ -566,18 +581,56 @@ class SX126x :
             else : self._status = self.STATUS_RX_DONE
             self._fixRxTimeout()
 
+    def listen(self, rxPeriod, sleepPeriod) :
+        self._irqSetup(self.IRQ_RX_DONE | self.IRQ_TIMEOUT | self.IRQ_HEADER_ERR | self.IRQ_CRC_ERR)
+        self._status = self.STATUS_RX_WAIT
+        rxPeriod = rxPeriod << 6
+        sleepPeriod = sleepPeriod << 6
+        if rxPeriod > 0x00FFFFFF : rxPeriod = 0x00FFFFFF
+        if sleepPeriod > 0x00FFFFFF : sleepPeriod = 0x00FFFFFF
+        if self._txen != -1 and self._rxen != -1 :
+            gpio.output(self._txen, gpio.LOW)
+            gpio.output(self._rxen, gpio.HIGH)
+        self._setRxDutyCycle(rxPeriod, sleepPeriod)
+        if self._irq != -1 :
+            self._statusInterrupt = self.STATUS_INT_INIT
+            try : gpio.add_event_detect(self._irq, gpio.RISING, callback=self._interruptRx, bouncetime=100)
+            except : pass
+        else :
+            irqStat = self._waitIrq()
+            payloadLengthRx = []; rxStartBufferPointer = []
+            self._getRxBufferStatus(payloadLengthRx, rxStartBufferPointer)
+            self._payloadTxRx = payloadLengthRx[0]
+            self._bufferIndex = rxStartBufferPointer[0]
+            if self._rxen != -1 : gpio.output(self._rxen, gpio.LOW)
+            if irqStat & self.IRQ_TIMEOUT : self._status = self.STATUS_RX_TIMEOUT
+            elif irqStat & self.IRQ_HEADER_ERR : self._status = self.STATUS_HEADER_ERR
+            elif irqStat & self.IRQ_CRC_ERR : self._status = self.STATUS_CRC_ERR
+            else : self._status = self.STATUS_RX_DONE
+            self._fixRxTimeout()
+
     def available(self) :
         return self._payloadTxRx
 
-    def read(self, length=1) :
+    def read(self, length=0) :
+        single = False
+        if length == 0 : length = 1; single = True
         buf = []
         self._readBuffer(self._bufferIndex, buf, length)
         self._bufferIndex = (self._bufferIndex + length) % 256
         if self._payloadTxRx > length : self._payloadTxRx -= length
         else : self._payloadTxRx = 0
-        if length == 1 : return buf[0]
+        if single : return buf[0]
         else : return buf
 
+    def get(self, length=1) :
+        buf = []
+        self._readBuffer(self._bufferIndex, buf, length)
+        self._bufferIndex = (self._bufferIndex + length) % 256
+        if self._payloadTxRx > length : self._payloadTxRx -= length
+        else : self._payloadTxRx = 0
+        return bytes(buf)
+
     def flush(self) :
         self._bufferIndex += self._payloadTxRx
         self._payloadTxRx = 0
@@ -707,14 +760,15 @@ class SX126x :
         self._writeBytes(0x9F, [enable], 1)
 
     def _setRxDutyCycle(self, rxPeriod, sleepPeriod) :
-        buf = []
-        buf[0] = (rxPeriod >> 16) & 0xFF
-        buf[1] = (rxPeriod >> 8) & 0xFF
-        buf[2] = rxPeriod & 0xFF
-        buf[0] = (sleepPeriod >> 16) & 0xFF
-        buf[1] = (sleepPeriod >> 8) & 0xFF
-        buf[2] = sleepPeriod & 0xFF
-        self._writeBytes(0x82, buf, 6)
+        buf = [
+            (rxPeriod >> 16) & 0xFF,
+            (rxPeriod >> 8) & 0xFF,
+            rxPeriod & 0xFF,
+            (sleepPeriod >> 16) & 0xFF,
+            (sleepPeriod >> 8) & 0xFF,
+            sleepPeriod & 0xFF
+        ]
+        self._writeBytes(0x94, buf, 6)
 
     def _setCad(self) :
         self._writeBytes(0xC5, [], 0)
@@ -852,15 +906,16 @@ class SX126x :
         self._writeBytes(0x8B, buf, 8)
 
     def _setModulationParamsFsk(self, br, pulseShape, bandwidth, Fdev) :
-        buf = []
-        buf[0] = (br >> 16) & 0xFF
-        buf[1] = (br >> 8) & 0xFF
-        buf[2] = br & 0xFF
-        buf[3] = pulseShape
-        buf[4] = bandwidth
-        buf[5] = (br >> 16) & 0xFF
-        buf[6] = (br >> 8) & 0xFF
-        buf[7] = Fdev & 0xFF
+        buf = [
+            (br >> 16) & 0xFF,
+            (br >> 8) & 0xFF,
+            br & 0xFF,
+            pulseShape,
+            bandwidth,
+            (br >> 16) & 0xFF,
+            (br >> 8) & 0xFF,
+            Fdev & 0xFF
+        ]
         self._writeBytes(0x8B, buf, 8)
 
     def _setPacketParamsLoRa(self, preambleLength, headerType, payloadLength, crcType, invertIq) :
@@ -883,29 +938,31 @@ class SX126x :
         self._writeBytes(0x8C, buf, 9)
 
     def _setPacketParamsFsk(self, preambleLength, preambleDetector, syncWordLength, addrComp, packetType, payloadLength, crcType, whitening) :
-        buf = []
-        buf[0] = (preambleLength >> 8) & 0xFF
-        buf[1] = preambleLength & 0xFF
-        buf[2] = preambleDetector
-        buf[3] = syncWordLength
-        buf[4] = addrComp
-        buf[5] = packetType
-        buf[6] = payloadLength
-        buf[7] = crcType
-        buf[8] = whitening
+        buf = [
+            (preambleLength >> 8) & 0xFF,
+            preambleLength & 0xFF,
+            preambleDetector,
+            syncWordLength,
+            addrComp,
+            packetType,
+            payloadLength,
+            crcType,
+            whitening
+        ]
         self._writeBytes(0x8C, buf, 9)
 
     def _setCadParams(self, cadSymbolNum, cadDetPeak, cadDetMin, cadExitMode, cadTimeout) :
         if cadSymbolNum < 0 or cadSymbolNum > 4 : return
         if cadExitMode < 0 or cadExitMode > 1 : return
-        buf = []
-        buf[0] = cadSymbolNum
-        buf[1] = cadDetPeak
-        buf[2] = cadDetMin
-        buf[3] = cadExitMode
-        buf[4] = (cadTimeout >> 16) & 0xFF
-        buf[5] = (cadTimeout >> 8) & 0xFF
-        buf[6] = cadTimeout & 0xFF
+        buf = [
+            cadSymbolNum,
+            cadDetPeak,
+            cadDetMin,
+            cadExitMode,
+            (cadTimeout >> 16) & 0xFF,
+            (cadTimeout >> 8) & 0xFF,
+            cadTimeout & 0xFF
+        ]
         self._writeBytes(0x88, buf, 7)
 
     def _setBufferBaseAddress(self, txBaseAddress, rxBaseAddress) :
